# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'E:\project\python\testplus_pc\mmgui\tools\..\mmgui\webview_window_ui.ui'
#
# Created by: PyQt5 UI code generator 5.12
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_WebViewWindowUI(object):
    def setupUi(self, WebViewWindowUI):
        WebViewWindowUI.setObjectName("WebViewWindowUI")
        WebViewWindowUI.resize(1098, 829)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/ic/resource/images/icon.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        WebViewWindowUI.setWindowIcon(icon)
        WebViewWindowUI.setStyleSheet("")
        self.containerWidget = QtWidgets.QWidget(WebViewWindowUI)
        self.containerWidget.setStyleSheet("")
        self.containerWidget.setObjectName("containerWidget")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.containerWidget)
        self.verticalLayout.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout.setSpacing(0)
        self.verticalLayout.setObjectName("verticalLayout")
        WebViewWindowUI.setCentralWidget(self.containerWidget)
        self.menubar = QtWidgets.QMenuBar(WebViewWindowUI)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 1098, 23))
        self.menubar.setObjectName("menubar")
        WebViewWindowUI.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(WebViewWindowUI)
        self.statusbar.setObjectName("statusbar")
        WebViewWindowUI.setStatusBar(self.statusbar)
        self.consoleLogDockWidget = QtWidgets.QDockWidget(WebViewWindowUI)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.consoleLogDockWidget.sizePolicy().hasHeightForWidth())
        self.consoleLogDockWidget.setSizePolicy(sizePolicy)
        self.consoleLogDockWidget.setMinimumSize(QtCore.QSize(200, 46))
        self.consoleLogDockWidget.setSizeIncrement(QtCore.QSize(0, 0))
        self.consoleLogDockWidget.setObjectName("consoleLogDockWidget")
        self.dockWidgetContents_4 = QtWidgets.QWidget()
        self.dockWidgetContents_4.setObjectName("dockWidgetContents_4")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.dockWidgetContents_4)
        self.verticalLayout_3.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_3.setSpacing(0)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.consoleLogDockWidget.setWidget(self.dockWidgetContents_4)
        WebViewWindowUI.addDockWidget(QtCore.Qt.DockWidgetArea(2), self.consoleLogDockWidget)
        self.actionDocument = QtWidgets.QAction(WebViewWindowUI)
        self.actionDocument.setObjectName("actionDocument")
        self.actionDeviceManagerView = QtWidgets.QAction(WebViewWindowUI)
        self.actionDeviceManagerView.setObjectName("actionDeviceManagerView")
        self.actionConsoleView = QtWidgets.QAction(WebViewWindowUI)
        self.actionConsoleView.setObjectName("actionConsoleView")
        self.actionWelcome = QtWidgets.QAction(WebViewWindowUI)
        self.actionWelcome.setObjectName("actionWelcome")
        self.actionSettings = QtWidgets.QAction(WebViewWindowUI)
        self.actionSettings.setObjectName("actionSettings")
        self.actionNew = QtWidgets.QAction(WebViewWindowUI)
        self.actionNew.setObjectName("actionNew")
        self.actionReportManager = QtWidgets.QAction(WebViewWindowUI)
        self.actionReportManager.setObjectName("actionReportManager")
        self.actionUpgrade = QtWidgets.QAction(WebViewWindowUI)
        self.actionUpgrade.setObjectName("actionUpgrade")
        self.actionAbout = QtWidgets.QAction(WebViewWindowUI)
        self.actionAbout.setObjectName("actionAbout")
        self.action_4 = QtWidgets.QAction(WebViewWindowUI)
        self.action_4.setObjectName("action_4")
        self.action_5 = QtWidgets.QAction(WebViewWindowUI)
        self.action_5.setObjectName("action_5")
        self.actionExit = QtWidgets.QAction(WebViewWindowUI)
        self.actionExit.setObjectName("actionExit")
        self.actionExportReport = QtWidgets.QAction(WebViewWindowUI)
        self.actionExportReport.setObjectName("actionExportReport")
        self.actionImportReport = QtWidgets.QAction(WebViewWindowUI)
        self.actionImportReport.setObjectName("actionImportReport")
        self.actionOpenReport = QtWidgets.QAction(WebViewWindowUI)
        self.actionOpenReport.setObjectName("actionOpenReport")
        self.actionReportComparerView = QtWidgets.QAction(WebViewWindowUI)
        self.actionReportComparerView.setObjectName("actionReportComparerView")
        self.actionConsole2View = QtWidgets.QAction(WebViewWindowUI)
        self.actionConsole2View.setObjectName("actionConsole2View")
        self.actionAccount = QtWidgets.QAction(WebViewWindowUI)
        self.actionAccount.setObjectName("actionAccount")

        self.retranslateUi(WebViewWindowUI)
        QtCore.QMetaObject.connectSlotsByName(WebViewWindowUI)

    def retranslateUi(self, WebViewWindowUI):
        _translate = QtCore.QCoreApplication.translate
        WebViewWindowUI.setWindowTitle(_translate("WebViewWindowUI", "mmgui"))
        self.consoleLogDockWidget.setWindowTitle(_translate("WebViewWindowUI", "Devtools"))
        self.actionDocument.setText(_translate("WebViewWindowUI", "文档"))
        self.actionDeviceManagerView.setText(_translate("WebViewWindowUI", "设备管理器"))
        self.actionConsoleView.setText(_translate("WebViewWindowUI", "调试控制台(js)"))
        self.actionWelcome.setText(_translate("WebViewWindowUI", "欢迎使用"))
        self.actionSettings.setText(_translate("WebViewWindowUI", "首选项"))
        self.actionNew.setText(_translate("WebViewWindowUI", "新建测试"))
        self.actionReportManager.setText(_translate("WebViewWindowUI", "报告管理器"))
        self.actionUpgrade.setText(_translate("WebViewWindowUI", "版本升级"))
        self.actionAbout.setText(_translate("WebViewWindowUI", "关于"))
        self.action_4.setText(_translate("WebViewWindowUI", "导入"))
        self.action_5.setText(_translate("WebViewWindowUI", "导出"))
        self.actionExit.setText(_translate("WebViewWindowUI", "退出"))
        self.actionExportReport.setText(_translate("WebViewWindowUI", "导出报告"))
        self.actionImportReport.setText(_translate("WebViewWindowUI", "导入报告"))
        self.actionOpenReport.setText(_translate("WebViewWindowUI", "打开本地报告"))
        self.actionReportComparerView.setText(_translate("WebViewWindowUI", "报告对比器"))
        self.actionConsole2View.setText(_translate("WebViewWindowUI", "调试控制台"))
        self.actionAccount.setText(_translate("WebViewWindowUI", "账户"))


from mmgui import res_rc



import logging
import traceback
from functools import wraps

from PyQt5.QtCore import QObject, pyqtSignal, QThreadPool, QRunnable, pyqtSlot

"""
asyncqt USAGE:
------------------------------------------------------
class DemoController(object):

    def on_refresh_btn_click(self):
        id = 1
        self._load_data_in_background(id)

    @run_on_worker_thread
    def _load_data_in_background(self, id):
        logging.info("_load_data_in_background start, id=" + str(id))
        time.sleep(1) # do some worker in background
        data = ["1", "2", "3"]
        logging.info("_load_data_in_background end")
        self._refresh_ui(data)

    @run_on_ui_thread
    def _refresh_ui(self, data):
        logging.info("_refresh_ui data: " + str(data))
------------------------------------------------------
"""


class UIThreadLoop(QObject):
    _msg_signal = pyqtSignal(tuple)

    def start_loop(self):
        self._msg_signal.connect(self._on_msg)

    def _on_msg(self, async_function):
        #logging.info("[UIThreadLoop] on msg, async_function=%s" % str(async_function))
        func = async_function[0]
        args = async_function[1]
        kwargs = async_function[2]
        func(*args, **kwargs)

    def _send_msg(self, async_function):
        #logging.info("[UIThreadLoop] send msg %s" % str(async_function))
        self._msg_signal.emit(async_function)

    def run_on_ui_thread(self, func, *args, **kwargs):
        async_function = (func, args, kwargs)
        self._send_msg(async_function)


class WorkerRunnable(QRunnable):

    def __init__(self, fn, *args, **kwargs):
        super(WorkerRunnable, self).__init__()
        self.fn = fn
        self.args = args
        self.kwargs = kwargs

    @pyqtSlot()
    def run(self):
        #logging.info("[WorkerRunnable] fn=%s, args=%s, kwargs=%s" % (str(self.fn), str(self.args), str(self.kwargs)))
        try:
            self.fn(*self.args, **self.kwargs)
        except Exception as e:
            traceback.print_exc()
            logging.exception(e)


class WorkerThreadExecutor(object):

    _thread_pool = QThreadPool()

    def __init__(self):
        self._thread_pool.setMaxThreadCount(20)
        self._thread_pool.setExpiryTimeout(30 * 1000) # default: 30seconds

    def execute(self, func, *args, **kwargs):
        runnable = WorkerRunnable(func, *args, **kwargs)
        self._thread_pool.start(runnable)


asyncqt_ui_thread_loop = UIThreadLoop()
asyncqt_worker_thread_executor = WorkerThreadExecutor()

def run_on_ui_thread(func):
    def wrapper(*args, **kwargs):
        asyncqt_ui_thread_loop.run_on_ui_thread(func, *args, **kwargs)
    return wrapper

def run_on_worker_thread(func):
    @wraps(func)
    def wrapper(*args, **kwargs):
        asyncqt_worker_thread_executor.execute(func, *args, **kwargs)
    return wrapper







import sys
# import signal

from typing import NoReturn, Callable

from PyQt5 import QtGui, QtCore
from PyQt5.QtCore import QCoreApplication, QSettings
from PyQt5.QtWidgets import QApplication, QSplashScreen

from .platform import setup_stdio, setup_console, run_as_job, STDOUT_STREAMS, STDERR_STREAMS
from .asyncqt import asyncqt_ui_thread_loop


class Context(object):
    pass



class App(Context):

    def __init__(self,
                 headless: bool = False,
                 icon_file = None,
                 splash_file = None,
                 splash_text = None,
                 configs_file = None,
                 log_file = None
                 ):
        self._headless = headless
        self._configs_file = configs_file
        self._icon_file = icon_file
        self._splash_file = splash_file
        self._splash_text = splash_text
        self._log_file = log_file
        self._settings : QSettings = None
        self._qt_application = None
        self._events_callback = {
            "create": [],
            "destroy": []
        }

    def on(self, event: str, callback: Callable[[Context], None]) -> NoReturn:
        if event not in self._events_callback:
            raise Exception("unsupported event %s" % event)
        self._events_callback[event].append(callback)

    def _notify_callback(self, event: str) -> NoReturn:
        if event not in self._events_callback:
            raise Exception("unsupported event %s" % event)
        for callback in self._events_callback[event]:
            callback(self)

    def on_create(self) -> NoReturn:
        self._notify_callback("create")

    def on_destroy(self) -> NoReturn:
        self._notify_callback("destroy")

    def run(self) -> int:
        setup_stdio()
        setup_console()
        run_as_job()

        argv = sys.argv[:]
        if self._headless:
            self._qt_application = QCoreApplication(argv)  # Non-GUI
            if sys.platform == 'linux':
                import signal
                signal.signal(signal.SIGINT, lambda *a: self._qt_application.quit())
        else:
            self._qt_application = QApplication(argv)

            # icon
            if self._icon_file:
                self._window_icon = QtGui.QIcon(self._icon_file)
                self._qt_application.setWindowIcon(self._window_icon)

            # splash
            if self._splash_file:
                pixmap = QtGui.QPixmap(self._splash_file)
                self._splash = QSplashScreen(pixmap)
                if self._splash_text:
                    self._set_splash_text(self._splash_text, "#ffffff")
                self._splash.show()

        # asyncqt
        asyncqt_ui_thread_loop.start_loop()

        # configs
        if self._configs_file:
            self._settings = QSettings(self._configs_file, QSettings.IniFormat)
            self._settings.sync()

        # log
        if self._log_file:
            if sys.platform == 'win32':
                logfp = open(self._log_file, 'w')
                STDERR_STREAMS.add(logfp)
                STDOUT_STREAMS.add(logfp)
            else:
                logfp = open(self._log_file, 'a')
                sys.stdout = logfp
                sys.stderr = logfp
        self._qt_application.aboutToQuit.connect(self._on_quit)
        self.on_create() # -> create and show the WebView window
        exit_code = self._qt_application.exec_()
        self._qt_application.deleteLater()
        return exit_code
        #sys.exit(exit_code)

    def _set_splash_text(self, text, color):
        self._splash.showMessage(text, alignment=QtCore.Qt.AlignHCenter | QtCore.Qt.AlignBottom, color=QtGui.QColor(color))

    def set_main_window(self, window):
        if window:
            self._splash.finish(window)
            window.setWindowIcon(self._window_icon)

    def get_config(self, key, def_val = None):
        if self._settings:
            return self._settings.value(key, def_val)
        return def_val

    def _on_quit(self):
        self.on_destroy()

    def exit(self) -> NoReturn:
        self._qt_application.quit()

    def get_application_dir_path(self):
        return self._qt_application.applicationDirPath()

    def get_headless(self):
        return self._headless



from mmgui import WebView, BrowserWindow


def test_load_url(qtbot):
    target_url = "http://www.baidu.com"
    webview = WebView(None, "testWebEngineView", True)
    qtbot.addWidget(webview.get_web_engine_view())
    with qtbot.waitCallback() as cb:
        def on_url_changed(url):
            print("on_page_load_finished")
            assert url == target_url
            cb(1)
        webview.register_event_listener("on_url_changed", on_url_changed)
        webview.load_url(target_url)
    cb.assert_called_with(1)
    webview.destroy()


def test_get_cookie(qtbot):
    webview = WebView(None, "testWebEngineView", True)
    qtbot.addWidget(webview.get_web_engine_view())
    with qtbot.waitCallback() as cb:
        def on_page_load_finished(event):
            print("on_page_load_finished")
            cookie = webview.get_cookie(".baidu.com", "BAIDUID")
            print("cookie: %s" % cookie)
            assert cookie is not None
            cb(1)
        webview.register_event_listener("on_page_load_finished", on_page_load_finished)
        webview.load_url("http://www.baidu.com")
    cb.assert_called_with(1)
    webview.destroy()


def test_run_javascript_code(qtbot):
    webview = WebView(None, "testWebEngineView", True)
    qtbot.addWidget(webview.get_web_engine_view())
    with qtbot.waitCallback(timeout=60*1000) as cb:
        def on_page_load_finished(event):
            print("on_page_load_finished")
            def callback(e):
                cb(1)
            webview.run_javascript_code("alert(1)", callback)
        webview.register_event_listener("on_page_load_finished", on_page_load_finished)
        webview.load_url("http://www.baidu.com")
    cb.assert_called_with(1)
    webview.destroy()


def test_show_alert_dialog(qtbot):
    win = BrowserWindow({})
    win.show()
    win.show_alert_dialog("title", "content")


def test_show_file_dialog_for_dir(qtbot):
    win = BrowserWindow({})
    win.show()
    win.show_file_dialog_for_dir("title")


def test_show_file_dialog_for_file(qtbot):
    win = BrowserWindow({})
    win.show()
    win.show_file_dialog_for_file("title", "*.txt")


def test_show_file_dialog_for_save_file(qtbot):
    win = BrowserWindow({})
    win.show()
    win.show_file_dialog_for_save_file("filename", "content", "*.txt")


from typing import Callable


class Menu(object):

    def __init__(self, title = None, on_click: Callable = None):
        self.title = title
        self.on_click = on_click
        self._children = []

    def append(self, menu):
        self._children.append(menu)

    def remove(self, menu):
        self._children.remove(menu)

    def children(self):
        return self._children


class MenuSeparator(Menu):

    def __init__(self):
        Menu.__init__(self)


import codecs
import json
import logging
import os
from typing import NoReturn, Callable, Any

from PyQt5 import QtCore, QtWidgets
from PyQt5.QtCore import QUrl, QObject, pyqtSignal, pyqtSlot, QVariant, QDir, QEvent, QPoint, QMimeData
from PyQt5.QtWidgets import QMainWindow, QWidget, QMessageBox, QFileDialog, QShortcut
from PyQt5.QtGui import QDragEnterEvent, QDragLeaveEvent, QDropEvent, QKeyEvent, QKeySequence, QCloseEvent
from PyQt5.QtWebEngineWidgets import QWebEngineView, QWebEngineScript, QWebEngineSettings
from PyQt5.QtWebChannel import QWebChannel

from .asyncqt import run_on_worker_thread
from .menu import MenuSeparator
from .webview_window_ui import Ui_WebViewWindowUI


g_default_configs = {
    "width": 800,
    "height": 600,
    "dev_mode": False,
    "frameless": False,
    "statusbar": False,
    "menu": None,
    "title": "mmgui"
}

logger = logging.getLogger("WebView")


class MyQWebEngineView(QWebEngineView):
    """
    QT doc:
    WebEngineView: https://doc.qt.io/qt-5/qwebengineview.html
    WebEnginePage: https://doc.qt.io/qt-5/qwebenginepage.html
    This version of Qt WebEngine is based on Chromium version 65.0.3325.151, with additional security fixes from newer versions.
    """

    def __init__(self, parent, drop_callback: Callable[[QDropEvent], None], dev_mode: bool):
        super(QWebEngineView, self).__init__(parent)
        self._drop_callback = drop_callback
        self._dev_mode = dev_mode

    def dragEnterEvent(self, event: QDragEnterEvent):
        logger.info("dragEnterEvent %s", event)
        event.acceptProposedAction()

    def dragLeaveEvent(self, event: QDragLeaveEvent):
        logger.info("dragLeaveEvent %s", event)

    def dropEvent(self, event: QDropEvent):
        logger.info("dropEvent %s", event)
        self._drop_callback(event)


class MyQMainWindow(QMainWindow):

    def __init__(self, parent, on_close_callback: Callable):
        super(QMainWindow, self).__init__(parent)
        self._on_close_callback = on_close_callback

    def closeEvent(self, event: QCloseEvent):
        if self._on_close_callback:
            self._on_close_callback()


class CookiesJar(object):

    def __init__(self):
        self._cookies = {}

    def add_cookie(self, cookie):
        domain = cookie.domain()
        name = cookie.name().data().decode("utf-8")
        value = cookie.value().data().decode("utf-8")
        logger.info("[WebView][CookiesJar] add cookie domain=%s, name=%s, value=%s", domain, name, value)
        if domain not in self._cookies:
            self._cookies[domain] = {}
        self._cookies[domain][name] = value

    def remove_cookie(self, cookie):
        domain = cookie.domain()
        name = cookie.name().data().decode("utf-8")
        logger.info("[WebView][CookiesJar] remove cookie domain=%s, name=%s", domain, name)
        try:
            if domain in self._cookies:
                del self._cookies[domain][name]
        except:
            pass

    def get_cookie(self, domain, name):
        value = None
        if domain in self._cookies and name in self._cookies[domain]:
            value = self._cookies[domain][name]
        logger.info("[WebView][CookiesJar] get cookie domain=%s, name=%s, value=%s", domain, name, value)
        return value


class WebViewBridge(QObject):

    on_message = pyqtSignal(str) # py -> js

    def __init__(self, webview_window):
        super().__init__()
        self._webview_window = webview_window
        self._function_map = {}

    def bind_function(self, js_function_name, py_function):
        print("bind_function %s " % js_function_name)
        self._function_map[js_function_name] = py_function

    def unbind_function(self, js_function_name):
        del self._function_map[js_function_name]

    @pyqtSlot(str, str, name='invoke', result=QVariant) # js -> py, sync call
    def invoke(self, function_name, params):
        if params:
            params = json.loads(params)
        else:
            params = {}
        result = None
        if function_name in self._function_map:
            result = self._function_map[function_name](**params)
        return result

    @pyqtSlot(str, str, str, name='post_message') # js -> py, ansync call(Callback or Promise)
    def js_post_message_to_py(self, callback_id, function_name, params):
        self.invoke_on_worker_thread(callback_id, function_name, params)

    @run_on_worker_thread
    def invoke_on_worker_thread(self, callback_id, function_name, params):
        result = self.invoke(function_name, params)
        self.py_reply_message_to_js(callback_id, result)

    def py_reply_message_to_js(self, callback_id, result):
        self.on_message.emit(json.dumps({ "callback_id": callback_id, "result": result}))

    def send_message(self, msg):
        #print("window python send message to js: %s" % str(msg))
        self.on_message.emit(json.dumps({ "callback_id": -1, "result": msg}))


class WebViewEvent(object):

    def __init__(self, type, data):
        self.type = type
        self.data = data


class BrowserWindow(object):

    def __init__(self, configs: dict):
        self._configs = {**g_default_configs, **configs}
        self._menus = []
        self._actions = []
        self._setup_ui()
        self._on_window_close_callback = None

    def _setup_ui(self) -> NoReturn:
        self._setup_main_window()
        self._setup_menus()
        self._setup_status_bar()
        self._setup_web_engine_view()
        self._setup_shortcut_keys()

    def _setup_main_window(self) -> NoReturn:
        self._main_window = MyQMainWindow(None, self._on_window_close)
        if self._configs['frameless']:
            self._main_window.setWindowFlags(QtCore.Qt.FramelessWindowHint)
        self._widget_ui = Ui_WebViewWindowUI()
        self._widget_ui.setupUi(self._main_window)
        self._main_window.setWindowTitle(self._configs['title'])
        if self._configs['width'] != -1 and self._configs['height'] != -1:
            self._main_window.resize(self._configs['width'], self._configs['height'])

    def get_main_window(self):
        return self._main_window

    def set_on_window_close_callback(self, callback):
        self._on_window_close_callback = callback

    def _on_window_close(self):
        if self._on_window_close_callback:
            self._on_window_close_callback()

    def _setup_menus(self) -> NoReturn:
        app_menu = self._configs['menu']
        if app_menu:
            menubar = self._widget_ui.menubar
            qt_menu = QtWidgets.QMenu(menubar)
            qt_menu.setObjectName("test menu")
            qt_menu.setTitle("test menu")

            for menu in app_menu.children():
                qt_menu = QtWidgets.QMenu(menubar)
                qt_menu.setObjectName(menu.title)
                qt_menu.setTitle(menu.title)
                menubar.addAction(qt_menu.menuAction())
                self._menus.append(qt_menu)
                for sub_menu in menu.children():
                    if sub_menu.on_click:
                        qt_action = QtWidgets.QAction(self._main_window)
                        qt_action.setObjectName(sub_menu.title)
                        qt_action.setText(sub_menu.title)
                        qt_action.triggered.connect(sub_menu.on_click)
                        self._actions.append(qt_action)
                        qt_menu.addAction(qt_action)
                    elif type(sub_menu) == MenuSeparator:
                        qt_menu.addSeparator()

            self._main_window.setMenuBar(menubar)
        else:
            self._main_window.setMenuBar(None)

    def _find_action(self, title):
        for action in self._actions:
            print("menu loop", action.objectName())
            if action.objectName() == title:
                return action
        return None

    def set_menu_visible(self, menu_title, visible):
        action = self._find_action(menu_title)
        if action:
            action.setVisible(visible)

    def _setup_status_bar(self) -> NoReturn:
        if not self._configs['statusbar']:
            self._main_window.setStatusBar(None)

    def _setup_web_engine_view(self) -> NoReturn:
        self.webview = WebView(self._main_window, "webEngineView", self._configs['dev_mode'])
        self._widget_ui.verticalLayout.addWidget(self.webview.get_widget_view())
        if self._configs['dev_mode']:
            self._devtools_web_view = WebView(self._main_window, "webDevEngineView", False)
            self._widget_ui.verticalLayout_3.addWidget(self._devtools_web_view.get_widget_view())
            self.webview.set_web_dev_tools_page(self._devtools_web_view.get_web_engine_view_page()) # bind webEngineView with devTools
            self._widget_ui.consoleLogDockWidget.setVisible(True)
        else:
            self._widget_ui.consoleLogDockWidget.setVisible(False)

    def set_style_sheet(self,  style_sheet_dir) -> NoReturn:
        self._main_window.setStyleSheet(style_sheet_dir)

    def _setup_shortcut_keys(self):
        self._shortcut_refresh = QShortcut(QKeySequence('Ctrl+R'), self._main_window)
        self._shortcut_refresh.activated.connect(self._on_refresh_shortcut_key_pressed)
        self._shortcut_refresh2 = QShortcut(QKeySequence('F5'), self._main_window)
        self._shortcut_refresh2.activated.connect(self._on_refresh_shortcut_key_pressed)
        self._shortcut_fullscreen = QShortcut(QKeySequence('F11'), self._main_window)
        self._shortcut_fullscreen.activated.connect(self._on_fullscreen_shortcut_key_pressed)
        self._shortcut_devtools = QShortcut(QKeySequence('F12'), self._main_window)
        self._shortcut_devtools.activated.connect(self._on_devtools_shortcut_key_pressed)

    def _on_refresh_shortcut_key_pressed(self):
        self.webview.reload()

    def _on_devtools_shortcut_key_pressed(self):
        if self._widget_ui.consoleLogDockWidget.isVisible():
            self._widget_ui.consoleLogDockWidget.hide()
        else:
            self._widget_ui.consoleLogDockWidget.show()

    def _on_fullscreen_shortcut_key_pressed(self):
        if self._main_window.isFullScreen():
            self._main_window.showNormal()
        else:
            self._main_window.showFullScreen()

    def show(self) -> NoReturn:
        if self._configs['width'] != -1 and self._configs['height'] != -1:
            self._main_window.show()
        else:
            self._main_window.showMaximized()

    def show_maximized(self) -> NoReturn:
        self._main_window.showMaximized()

    def show_minimized(self) -> NoReturn:
        self._main_window.showMinimized()

    def show_normal(self) -> NoReturn:
        self._main_window.showNormal()

    def show_full_screen(self) -> NoReturn:
        self._main_window.showFullScreen()

    def is_maximized(self) -> bool:
        return self._main_window.windowState() == QtCore.Qt.WindowMaximized

    def is_full_screen(self) -> bool:
        return self._main_window.windowState() == QtCore.Qt.WindowFullScreen

    def move(self, x, y):
        self._main_window.move(x, y)

    def move_by(self, dx, dy):
        cur_pos: QPoint = self._main_window.pos()
        self._main_window.move(cur_pos.x() + dx, cur_pos.y() + dy)

    def close(self) -> NoReturn:
        if self.webview:
            self.webview.destroy()
        self._main_window.close()

    def show_alert_dialog(self, title: str, msg: str, cancelable: bool = True) -> bool:
        message_box = QMessageBox()
        message_box.setWindowTitle(title)
        message_box.setIcon(QMessageBox.Question)
        message_box.setText(msg)
        message_box.setStandardButtons((QMessageBox.Yes | QMessageBox.Cancel) if cancelable else QMessageBox.Yes)
        result = message_box.exec()
        return result == QMessageBox.Yes

    def show_file_dialog_for_save_file(self, filename: str, msg: str, filter: str) -> list:
        options = QFileDialog.Options()
        files = QFileDialog.getSaveFileName(None,
                                            msg,
                                            filename,
                                            filter=filter,
                                            options=options)
        return files

    def show_file_dialog_for_file(self, title: str, filter: str) -> list:
        options = QFileDialog.Options()
        files, _ = QFileDialog.getOpenFileNames(None,
                                                title,
                                                directory=QDir.homePath(),
                                                filter=filter,
                                                options=options)
        return files

    def show_file_dialog_for_dir(self, title: str) -> str:
        options = QFileDialog.Options()
        choose_dir = QFileDialog.getExistingDirectory(None,
                                                      title,
                                                      directory=QDir.homePath(),
                                                      options=options)
        return choose_dir


class WebView(object):

    def __init__(self, parent, object_name, dev_mode: bool):
        self._event_listeners = {}
        self._web_engine_view = None
        self._dev_mode = dev_mode
        self._object_name = object_name
        self._setup_web_engine_view(parent)
        self.move_window_func = None

    def _setup_web_engine_view(self, parent) -> NoReturn:
        self._web_engine_view = self._create_web_engine_view(parent, self._object_name)
        self._web_engine_view.urlChanged.connect(self._on_url_changed)
        self._web_engine_view.page().loadFinished.connect(self._on_page_load_finished)

        # cookies
        self._cookiesJar = CookiesJar()
        self._web_engine_view.page().profile().cookieStore().cookieAdded.connect(self._on_cookie_added)
        self._web_engine_view.page().profile().cookieStore().cookieRemoved.connect(self._on_cookie_removed)

        # web channel
        self._web_channel = QWebChannel(self._web_engine_view.page())
        self._web_engine_view.page().setWebChannel(self._web_channel)
        self._web_bridge = WebViewBridge(self)
        self._web_channel.registerObject("proxy", self._web_bridge)

        # inject scripts
        self.inject_javascript_file(os.path.join(os.path.dirname(os.path.abspath(__file__)), "res", "js", "qwebchannel.js"))
        self.inject_javascript_file(os.path.join(os.path.dirname(os.path.abspath(__file__)), "res", "js", "mmgui.js"))

    def register_web_channel_object(self, namespace: str, object: QObject) -> NoReturn:
        self._web_channel.registerObject(namespace, object)

    def get_web_engine_view(self):
        return self._web_engine_view

    def get_web_engine_view_page(self):
        return self._web_engine_view.page()

    def get_widget_view(self):
        return self._web_engine_view

    def register_event_listener(self, event_type: str, listener: Callable[[WebViewEvent], None]) -> NoReturn:
        if event_type not in self._event_listeners:
            self._event_listeners[event_type] = []
        self._event_listeners[event_type].append(listener)

    def unregister_event_listener(self, event_type: str, listener: Callable[[WebViewEvent], None]) -> NoReturn:
        if event_type in self._event_listeners and self._event_listeners[event_type]:
            for item in self._event_listeners[event_type]:
                if item == listener:
                    self._event_listeners[event_type].remove(item)
                    break

    def _notify_event_listeners(self, event: WebViewEvent):
        event_type = event.type
        if event_type in self._event_listeners and self._event_listeners[event_type]:
            for listener in self._event_listeners[event_type]:
                listener(event)

    def _on_url_changed(self, qurl):
        logger.info("_on_url_changed %s", qurl.toString())
        self._notify_event_listeners(WebViewEvent("on_url_changed", qurl.toString()))

    def _on_page_load_finished(self, ok):
        logger.info("_on_page_load_finished %s", ok)
        self._notify_event_listeners(WebViewEvent("on_page_load_finished", ok))

    def _on_cookie_added(self, cookie):
        logger.info("_on_cookie_added %s", cookie)
        self._cookiesJar.add_cookie(cookie)

    def _on_cookie_removed(self, cookie):
        logger.info("_on_cookie_removed %s", cookie)
        self._cookiesJar.remove_cookie(cookie)

    def delete_all_cookies(self):
        self._web_engine_view.page().profile().cookieStore().deleteAllCookies()

    def get_cookie(self, domain, name):
        return self._cookiesJar.get_cookie(domain, name)

    def _create_web_engine_view(self, parent, object_name: str) -> QWebEngineView :
        web_engine_view = MyQWebEngineView(parent, lambda e: self._on_drop(e), self._dev_mode)
        settings = QWebEngineSettings.defaultSettings()
        settings.setAttribute(QWebEngineSettings.LocalContentCanAccessRemoteUrls, True)
        web_engine_view.setAutoFillBackground(False)
        web_engine_view.setStyleSheet("")
        web_engine_view.setUrl(QtCore.QUrl("about:blank"))
        web_engine_view.setObjectName(object_name)
        web_engine_view.setContextMenuPolicy(QtCore.Qt.CustomContextMenu) # 禁用右键菜单
        return web_engine_view

    def _on_drop(self, event: QDropEvent) -> NoReturn:
        pos = event.pos()
        mime_data = event.mimeData() # type:QMimeData
        files = []
        if mime_data.hasUrls():
            urls = mime_data.urls()
            for url in urls:
                files.append(url.toLocalFile())
        logging.info("files %s %s", files, pos)
        if self._web_bridge:
            event = {
                "files": files,
                "pos": [pos.x(), pos.y()]
            }
            self.send_message_to_js({"type": "onDrop", "event": event})

    def bind_function(self, js_function_name: str, py_function: Callable) -> NoReturn:
        self._web_bridge.bind_function(js_function_name, py_function)

    def send_message_to_js(self, msg: Any) -> NoReturn:
        self._web_bridge.send_message(msg)

    def run_javascript_code(self, javascript_code: str, callback: Callable[[Any], None]) -> NoReturn:
        self._web_engine_view.page().runJavaScript(javascript_code, callback)

    def inject_javascript_file(self, javascript_file: str) -> NoReturn:
        if not os.path.exists(javascript_file):
            raise Exception("javascript file is not exists, filename=%s" % javascript_file)

        script = QWebEngineScript()
        with codecs.open(javascript_file, "r", "utf-8") as f:
            script.setSourceCode(f.read())
        script.setName(os.path.basename(javascript_file))
        script.setWorldId(QWebEngineScript.MainWorld)
        script.setInjectionPoint(QWebEngineScript.DocumentCreation)
        script.setRunsOnSubFrames(False)

        profile = self._web_engine_view.page().profile() # type: QWebEngineProfile
        scripts = profile.scripts() # type: QWebEngineScriptCollection
        scripts.insert(script)
        logger.info("inject javascript file %s" % javascript_file)

    def load_file(self, filename: str) -> NoReturn:
        self._web_engine_view.load(QUrl.fromLocalFile(filename))

    def load_url(self, url: str) -> NoReturn:
        self._web_engine_view.load(QUrl(url))

    def set_web_dev_tools_page(self, page) -> NoReturn:
        self._web_engine_view.page().setDevToolsPage(page)

    def reload(self):
        self._web_engine_view.reload()

    def destroy(self):
        self._web_engine_view.urlChanged.disconnect()
        self._web_engine_view.deleteLater()
        self._web_engine_view.close()





import os
import subprocess
import sys
import logging
from subprocess import list2cmdline
logger = logging.getLogger(__file__)

ORIGINAL_STDIO = {

}
STDOUT_STREAMS = set()
STDERR_STREAMS = set()

if sys.platform == 'win32':
    from _ctypes import Structure, POINTER
    from ctypes import *
    from ctypes.wintypes import *

    class PROCESSENTRY32(Structure):
        _fields_ = [
            ('dwSize', DWORD),
            ('cntUsage', DWORD),
            ('th32ProcessID', DWORD),
            ('th32DefaultHeapID', POINTER(ULONG)),
            ('th32ModuleID', DWORD),
            ('cntThreads', DWORD),
            ('th32ParentProcessID', DWORD),
            ('pcPriClassBase', LONG),
            ('dwFlags', DWORD),
            ('szExeFile', c_char * MAX_PATH)
        ]

    psapi = windll.psapi

    CreateToolhelp32Snapshot= windll.kernel32.CreateToolhelp32Snapshot
    Process32First = windll.kernel32.Process32First
    Process32Next = windll.kernel32.Process32Next
    GetLastError = windll.kernel32.GetLastError
    CloseHandle = windll.kernel32.CloseHandle
    TH32CS_SNAPPROCESS = 0x2

    class Process:

        def __init__(self, pid):
            self.pid = pid
            self.ppid = 0
            self.name = None
            self.children = {}

        def set_name(self, name):
            self.name = name

        def set_parent(self, ppid):
            self.ppid = ppid

        def add_child(self, process):
            if process.pid != self.pid:
                self.children[process.pid] = process

        def print(self, layer = 0):
            print("%s+ %d %s" % (' ' * layer, self.pid, self.name if self.name else ""))
            for c in self.children.values():
                c.print(layer + 1)

        def kill(self):
            for c in self.children.values():
                c.kill()
            os.kill(self.pid, 0)

    def get_self_process_tree():

        flat_map = {}

        holder = PROCESSENTRY32()
        holder.dwSize = sizeof(holder)

        hProcessSnap = CreateToolhelp32Snapshot(TH32CS_SNAPPROCESS, os.getpid())
        #print("hProcessEnum", hProcessSnap)

        ret = Process32First(hProcessSnap, pointer(holder))
        while ret:
            pid, ppid = holder.th32ProcessID, holder.th32ParentProcessID
            #print("pid=%d, ppid=%d %s" % (pid, ppid, holder.szExeFile))

            if pid not in flat_map: flat_map[pid] = Process(pid)
            if ppid not in flat_map: flat_map[ppid] = Process(ppid)
            flat_map[pid].set_parent(ppid)
            # flat_map[pid].set_name(holder.szExeFile.decode('gbk'))
            flat_map[ppid].add_child(flat_map[pid])

            ret = Process32Next(hProcessSnap, pointer(holder))

        CloseHandle(hProcessSnap)

        if 0 not in flat_map: flat_map[0] = Process(0)
        for pid in flat_map:
            if flat_map[pid].ppid == 0 and pid not in flat_map[0].children:
                flat_map[0].add_child(flat_map[pid])

        return flat_map[os.getpid()]

    CREATE_BREAKAWAY_FROM_JOB = 0x01000000
    ERROR_ALREADY_EXISTS = 183
    ERROR_ACCESS_DENIED = 5

    class JOBOBJECT_BASIC_LIMIT_INFORMATION(Structure):
        _fields_ = [
            ('PerProcessUserTimeLimit', c_uint64),
            ('PerJobUserTimeLimit', c_uint64),
            ('LimitFlags', c_uint32),
            ('MinimumWorkingSetSize', c_uint64),
            ('MaximumWorkingSetSize', c_uint64),
            ('ActiveProcessLimit', c_uint32),
            ('Affinity', c_uint64),
            ('PriorityClass', c_uint32),
            ('SchedulingClass', c_uint32)
        ]


    class JOBOBJECT_EXTENDED_LIMIT_INFORMATION(Structure):
        _fields_ = [
            ('BasicLimitInformation', JOBOBJECT_BASIC_LIMIT_INFORMATION),
            ('_reserve1', c_char * 0x30),
            ('ProcessMemoryLimit', c_uint64),
            ('JobMemoryLimit', c_uint64),
            ('PeakProcessMemoryUsed', c_uint64),
            ('PeakJobMemoryUsed', c_uint64)
        ]

    class STARTUPINFOW(Structure):
      _fields_ = [
        ("cb", DWORD),
        ("lpReserved",LPWSTR),
        ("lpDesktop", LPWSTR),
        ("lpTitle", LPWSTR),
        ("dwX", DWORD),
        ("dwY", DWORD),
        ("dwXSize", DWORD),
        ("dwYSize", DWORD),
        ("dwXCountChars", DWORD),
        ("dwYCountChars", DWORD),
        ("dwFillAttribute", DWORD),
        ("dwFlags", DWORD),
        ("wShowWindow", WORD),
        ("cbReserved2", WORD),
        ("lpReserved2", LPBYTE),
        ("hStdInput", HANDLE),
        ("hStdOutput", HANDLE),
        ("hStdError", HANDLE)
      ]

    class PROCESS_INFORMATION(Structure):
      _fields_ = [
        ("hProcess", HANDLE),
        ("hThread", HANDLE),
        ("dwProcessId", DWORD),
        ("dwThreadId", DWORD)
      ]

    class SECURITY_ATTRIBUTES(Structure):
      _fields_ = [
        ("nLength", DWORD),
        ("lpSecurityDescriptor", c_void_p),
        ("bInheritHandle", BOOL)
      ]

    JOB_OBJECT_LIMIT_KILL_ON_JOB_CLOSE = 0x00002000
    JOB_OBJECT_LIMIT_BREAKAWAY_OK = 0x800

    JobObjectBasicLimitInformation = 2
    JobObjectExtendedLimitInformation = 9

    CreateJobObjectW = windll.kernel32.CreateJobObjectW
    CreateJobObjectW.restype = HANDLE
    CreateJobObjectW.argtypes = [c_void_p, c_wchar_p]

    GetCurrentProcess = windll.kernel32.GetCurrentProcess
    GetCurrentProcess.restype = HANDLE

    AssignProcessToJobObject = windll.kernel32.AssignProcessToJobObject
    AssignProcessToJobObject.restype = BOOL
    AssignProcessToJobObject.argtypes = [HANDLE, HANDLE]

    OpenJobObjectW = windll.kernel32.OpenJobObjectW
    TerminateJobObject = windll.kernel32.TerminateJobObject
    TerminateJobObject.restype = BOOL
    TerminateJobObject.argtypes = [HANDLE, c_uint]

    SetInformationJobObject = windll.kernel32.SetInformationJobObject
    SetInformationJobObject.restype = BOOL
    SetInformationJobObject.argtypes = [HANDLE, c_int, POINTER(JOBOBJECT_EXTENDED_LIMIT_INFORMATION), DWORD]

    CreateProcessW = windll.kernel32.CreateProcessW
    CreateProcessW.restype = BOOL
    CreateProcessW.argtypes = [
      c_wchar_p,                        # lpApplicationName
      c_wchar_p,                        # lpCommandLine
      c_void_p,                         # lpProcessAttributes
      c_void_p,                         # lpThreadAttributes
      BOOL,                             # bInheritHandles
      DWORD,                            # dwCreationFlags
      c_void_p,                         # lpEnvironment
      c_wchar_p,                        # lpCurrentDirectory
      POINTER(STARTUPINFOW),            # lpStartupInfo
      POINTER(PROCESS_INFORMATION)      # lpProcessInformation
      ]

    CreateMutexW = windll.kernel32.CreateMutexA
    CreateMutexW.restype = HANDLE
    CreateMutexW.argtypes = [
      POINTER(SECURITY_ATTRIBUTES), # null
      BOOL,
      c_wchar_p
    ]

    OpenMutexW = windll.kernel32.OpenMutexW
    OpenMutexW.restype = HANDLE
    OpenMutexW.argtypes = [
      DWORD,
      BOOL,
      c_wchar_p
    ]

    ReleaseMutex = windll.kernel32.ReleaseMutex
    ReleaseMutex.argtypes = [HANDLE]

    CloseHandle = windll.kernel32.CloseHandle
    CloseHandle.argtypes = [HANDLE]

    WaitForSingleObject = windll.kernel32.WaitForSingleObject
    WaitForSingleObject.restype = DWORD
    WaitForSingleObject.argtypes = [
      HANDLE,
      DWORD
    ]

    ShellExecuteW = windll.shell32.ShellExecuteW
    ShellExecuteW.argtypes = [c_void_p, c_wchar_p,  c_wchar_p, c_wchar_p, c_wchar_p, c_int]

    def run_as_job():
        hProcess = GetCurrentProcess()
        hJob = CreateJobObjectW(None, "Global\\testplus_perftool_job_obj")
        if hJob == 0:
            print("CreateJobObjectW Failed:", GetLastError())
            return
        info = JOBOBJECT_EXTENDED_LIMIT_INFORMATION()
        info.BasicLimitInformation.LimitFlags = JOB_OBJECT_LIMIT_KILL_ON_JOB_CLOSE | JOB_OBJECT_LIMIT_BREAKAWAY_OK
        # print(sizeof(info))
        if not SetInformationJobObject(hJob, JobObjectExtendedLimitInformation, pointer(info), sizeof(info)):
            print("SetInformationJobObject Failed:", GetLastError())
            return
        if not AssignProcessToJobObject(hJob, hProcess):
            print("AssignProcessToJobObject Failed:", GetLastError())
            return
        # print("ok")
        return hJob

    MUTEX_NAME = "Global\\TestPlusPerformanceToolkit_Is_Running"

    def _self_restart_normal():
        # print([sys.executable] + sys.argv)
        cmd = [sys.executable] + sys.argv
        try:
            logging.info("self_restart: %r", cmd)
            subprocess.Popen(cmd, creationflags=subprocess.CREATE_BREAKAWAY_FROM_JOB)
            logger.info("self_restart ok!")
        except:
            logger.exception("self_restart")
            return False
        return True

    def _self_restart_admin():
        params = subprocess.list2cmdline(sys.argv)
        process = ShellExecuteW(
            None,  # HWND hwnd
            u"runas",  # LPCWSTR lpOperation
            sys.executable,  # LPCWSTR lpFile
            params,  # LPCWSTR lpParameters
            None,  # LPCWSTR lpDirectory,
            1  # INT nShowCmd, SEE_MASK_NO_CONSOLE|SEE_MASK_NOCLOSE_PROCESS )
        )
        return process > 32


    def self_restart(admin=False):
        if admin:
            return _self_restart_admin()
        else:
            return _self_restart_normal()

    def wait_for_previous_process():
        lock = CreateMutexW(None, False, MUTEX_NAME)
        err = GetLastError()
        if err == ERROR_ALREADY_EXISTS:
            WaitForSingleObject(lock, -1)
            return lock
        else:
            WaitForSingleObject(lock, -1)
        return lock


    GetConsoleWindow = windll.kernel32.GetConsoleWindow
    ShowWindow = windll.user32.ShowWindow
    AllocConsole = windll.kernel32.AllocConsole
    AttachConsole = windll.kernel32.AttachConsole
    AttachConsole.argtypes = [
      DWORD
    ]
    GetCurrentProcessId = windll.kernel32.GetCurrentProcessId

    IS_CONSOLE_HIDDEN = False

    def _hide_nothing():
        pass
      
    def _hide_console():
        ShowWindow(GetConsoleWindow(), 0)
        global IS_CONSOLE_HIDDEN
        IS_CONSOLE_HIDDEN = True

    WINDOW_OP_PROXY = {
        "hide": _hide_nothing
    }


    class StreamMux(object):
        # Based on https://gist.github.com/327585 by Anand Kunal
        # Based on http://www.tentech.ca/2011/05/stream-tee-in-python-saving-stdout-to-file-while-keeping-the-console-alive/ by Tennessee
        def __init__(self, arr):
            self.arr = arr
            self.attrs = {}

        def __getattribute__(self, name):
            return object.__getattribute__(self, name)

        def __getattr__(self, name):
            def mk_attr(name):
                def attr_func(*args, **kwargs):
                    ret = None
                    for stream in self.arr:
                        callable = getattr(stream, name)
                        ret = callable(*args, **kwargs)
                    return ret

                return attr_func

            # Could also be a property
            if name not in self.attrs:
                self.attrs[name] = mk_attr(name)
            return self.attrs[name]


    def hide_console():
        WINDOW_OP_PROXY['hide']()
      
    def show_console():
        ShowWindow(GetConsoleWindow(), 5)
        global IS_CONSOLE_HIDDEN
        IS_CONSOLE_HIDDEN = False

    def toggle_console():
        if IS_CONSOLE_HIDDEN:
            show_console()
        else:
            hide_console()


    def setup_console():
        if AllocConsole():
            AttachConsole(GetCurrentProcessId())
            fp = open("CONOUT$", "w")
            STDOUT_STREAMS.add(fp)
            STDERR_STREAMS.add(fp)
            sys.stdin = open("CONIN$", "r")
            WINDOW_OP_PROXY['hide'] = _hide_console
            hide_console()
            return True
        else:
            STDOUT_STREAMS.add(ORIGINAL_STDIO['stdout'])
            STDERR_STREAMS.add(ORIGINAL_STDIO['stderr'])
            return False

    def setup_stdio():
        assert not ORIGINAL_STDIO
        ORIGINAL_STDIO['stdout'], ORIGINAL_STDIO['stderr'] = sys.stdout, sys.stderr
        sys.stdout, sys.stderr = StreamMux(STDOUT_STREAMS), StreamMux(STDERR_STREAMS)
else:
    def setup_stdio():
        pass

    def setup_console():
        pass

    def wait_for_previous_process():
        pass

    def run_as_job():
        pass
    
    def hide_console():
        pass

    



# coding: utf-8
import os, re
from setuptools import setup, find_packages, find_namespace_packages

with open(os.path.join("mmgui", "__init__.py"), encoding="utf8") as f:
  version = re.search(r'__version__ = "(.*?)"', f.read()).group(1)

setup(
  name='mmgui',
  version=version,
  python_requires='>=3.6',
  description='mmgui',
  url='http://gitlab.testplus.cn/sandin/mmgui',
  author='lds2012',
  author_email='lds2012@gmail.com',
  license='GPLv3',
  include_package_data=True, # 代码以外的文件也包含进来, 这些文件可以通过 __file__ 加相对路径找到
  packages=find_namespace_packages(include=['mmgui.*', "mmgui"]),
  install_requires='''
PyQt5==5.12
PyQt5-sip==4.19.19
PyQtWebEngine==5.12
'''.split('\n'),
  zip_safe=False)
